package cn.com.ty.lift.business.framework.job;

import cn.com.ty.lift.business.maintenance.dao.entity.MaintenancePlan;
import cn.com.ty.lift.business.maintenance.service.MaintenancePlanService;
import cn.com.ty.lift.business.push.entity.InspectionTips;
import cn.com.ty.lift.business.push.entity.RepairTips;
import cn.com.ty.lift.business.push.entity.TaskTips;
import cn.com.ty.lift.business.push.service.PushUserService;
import cn.com.ty.lift.common.model.PushMessage;
import cn.com.ty.lift.common.model.PushUserInfo;
import cn.hutool.core.collection.IterUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.jms.core.JmsMessagingTemplate;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 定时任务
 *
 * @author wcz
 * @since 2020/1/18 17:16
 */
@Slf4j
@Component
@EnableScheduling
@AllArgsConstructor
public class BusinessJob {

    private MaintenancePlanService maintenancePlanService;
    private JmsMessagingTemplate   jmsMessagingTemplate;
    private PushUserService        pushUserService;

    /**
     * 更新维保计划过期状态job：每天1-3点，每10分钟执行一次
     */
    @Scheduled(cron = "0 0/10 1-3 * * ?")
    public void updateMtPlanJob() {
        log.warn("Starting update maintenance plan overdue job...");
        long pageNum = 1;
        long pages;
        do {
            IPage<MaintenancePlan> planPages = maintenancePlanService.pageOverduePlan(new Page<>(pageNum, 100));
            pages = planPages.getPages();
            List<MaintenancePlan> records = planPages.getRecords();
            if (!records.isEmpty()) {
                records.forEach(record -> {
                    record.setStatus(-1);
                });
                maintenancePlanService.updateBatchById(records);
            }
            pageNum++;
        } while (pageNum <= pages);
    }

    /**
     * 年检到期提前45天提醒
     */
    @Scheduled(cron = "0 0 9 * * ?")
    public void inspection45days() {
        log.warn("Starting inspection 45days notice job...");
        sendAnnualInspection(45);
    }

    private void sendAnnualInspection(int days) {
        long pageNum = 1;
        long pages;
        do {
            IPage<InspectionTips> tips = pushUserService.pageMtCompanyByAnnualInspection(new Page<>(pageNum, 10), days);
            pages = tips.getPages();
            List<InspectionTips> records = tips.getRecords();
            if(records.isEmpty()){
                continue;
            }
            if (days == 45) {
                for (InspectionTips record : records) {
                    List<PushUserInfo> pushUserInfos = pushUserService.listByUserIds(record.getMtCompanyId(), record.members());
                    if (IterUtil.isNotEmpty(pushUserInfos)) {
                        PushMessage.inspectionAdvance15daysNotice(record.getProjectName(), record.getDevicePosition(), record.getRegistrationCode()).sendTokenOnPlatform(jmsMessagingTemplate, pushUserInfos);
                    }
                }
            } else {
                for (InspectionTips record : records) {
                    List<PushUserInfo> pushUserInfos = pushUserService.listByUserIds(record.getMtCompanyId(), record.members());
                    if (IterUtil.isNotEmpty(pushUserInfos)) {
                        PushMessage.inspectionAdvance15daysNotice(record.getProjectName(), record.getDevicePosition(), record.getRegistrationCode()).sendTokenOnPlatform(jmsMessagingTemplate, pushUserInfos);
                    }
                }
            }
            pageNum++;
        } while (pageNum <= pages);
    }
    /**
     * 年检到期提前15天提醒
     */
//    @Scheduled(cron = "0/30 * * * * ?")
    @Scheduled(cron = "0 0 10 * * ?")
    public void inspection15days() {
        log.warn("Starting inspection 15days notice job...");
        sendAnnualInspection(15);
    }

    @Scheduled(cron = "0 0 16 * * ?")
    public void taskNotice() {
        log.trace("Starting task report notice job...");
        List<RepairTips> repairTips = pushUserService.listMtCompanyByEmergencyRepair();
        if(repairTips.isEmpty()){
            return;
        }
        //按维保公司分组
        Map<Long, List<RepairTips>> companies = repairTips.stream().collect(Collectors.groupingBy(RepairTips::getMtCompanyId));
        if(companies.isEmpty()){
            return;
        }
        List<TaskTips> taskTips = pushUserService.listMaintainForEnterpriseAdmin();
        Map<Long, List<TaskTips>> maintains = taskTips.stream().collect(Collectors.groupingBy(TaskTips::getMtCompanyId));
        for (Map.Entry<Long, List<RepairTips>> entry : companies.entrySet()) {
            int companyTotal = 0, companyFinish = 0, companyDoings = 0;
            List<RepairTips> repairTipsList = entry.getValue();
            if(IterUtil.isNotEmpty(repairTipsList)){
                List<TaskTips> tipsList = maintains.get(repairTipsList.get(0).getMtCompanyId());
                int maintain = 0;
                if(IterUtil.isNotEmpty(tipsList)){
                    maintain = tipsList.get(0).getMaintainTotal();
                }
                //按区域分组, 发给区域主管
                Map<String, List<RepairTips>> areas = repairTipsList.stream().collect(Collectors.groupingBy(RepairTips::getAreaName));
                for (Map.Entry<String, List<RepairTips>> listEntry : areas.entrySet()) {
                    List<RepairTips> tips = listEntry.getValue();
                    int total = 0, finish = 0, doings = 0;
                    //循环同一个公司同一个区域下的数据
                    for (RepairTips tip : tips) {
                        total += tip.getTotal();
                        if(tip.isRepairing()){
                            doings += tip.getTotal();
                        }else if(tip.isFinish()){
                            finish += tip.getTotal();
                        }
                    }
                    companyTotal += total;
                    companyFinish += finish;
                    companyDoings += doings;

                    RepairTips repair = tips.get(0);
                    //区域主管（项目所属）
                    PushUserInfo director = pushUserService.listByUserId(repair.getMtCompanyId(), repair.getDirectorId());
                    PushMessage.managerTaskForRegionChargerDaily(repair.getAreaName(), total, finish, doings, maintain).sendTokenOnPlatform(jmsMessagingTemplate, director);
                }
                //总经理
                List<PushUserInfo> pushUserInfos = pushUserService.listHighDirector(repairTipsList.get(0).getMtCompanyId());
                PushMessage.managerTaskForLeaderDaily(companyTotal, companyFinish, companyDoings, maintain).sendTokenOnPlatform(jmsMessagingTemplate, pushUserInfos);
            }
        }
    }

    @Scheduled(cron = "0 0 8 * * ?")
    public void pageRepairAndMaintainForClerk() {
        log.info("Staring repair and maintain for clerk job...");
        long pageNum = 1;
        long pages;
        do {
            IPage<TaskTips> tipsIPage = pushUserService.pageRepairAndMaintainForClerk(new Page<>(pageNum, 20));
            pages = tipsIPage.getPages();
            List<TaskTips> records = tipsIPage.getRecords();
            if (!records.isEmpty()) {
                for (TaskTips record : records) {
                    PushUserInfo pushUserInfo = pushUserService.listByUserId(record.getMtCompanyId(), record.getUserId());
                    if(Objects.nonNull(pushUserInfo)){
                        PushMessage.managerMaintainDaily(record.getMaintainTotal(), record.getRepairTotal());
                    }
                }
            }
            pageNum++;
        } while (pageNum <= pages);
    }
}
